var Uri = android.net.Uri;
import { ParameterBuilder } from './parameterBuilder';
import { UserInfo } from '../../common/userInfo';
import { DatabaseUser } from '../../common/databaseUser';
import { Credentials } from '../../common/credentials';
import { RequestFactory } from '../request/internal/requestFactory';
import { AuthenticationErrorBuilder } from '../request/internal/authenticationErrorBuilder';
import { DatabaseConnectionRequest } from './request/databaseConnectionRequest';
import { ProfileRequest } from './request/profileRequest';
import { SignUpRequest } from './request/signUpRequest';
import { TokenRequest } from './request/tokenRequest';
export class AuthenticationAPIClient {
    constructor(auth0, factory = new RequestFactory()) {
        this.auth0 = auth0;
        this.factory = factory;
        this.authErrorBuilder = new AuthenticationErrorBuilder();
        const telemetry = auth0.getTelemetry();
        if (telemetry != null) {
            factory.setClientInfo(telemetry.getValue());
        }
    }
    getClientId() {
        return this.auth0.getClientId();
    }
    getBaseURL() {
        return this.auth0.getDomainUrl();
    }
    setUserAgent(userAgent) {
        this.factory.setUserAgent(userAgent);
    }
    login(usernameOrEmail, password, realmOrConnection) {
        const builder = ParameterBuilder.newBuilder()
            .setParameter(AuthenticationAPIClient.USERNAME_KEY, usernameOrEmail)
            .setParameter(AuthenticationAPIClient.PASSWORD_KEY, password);
        if (realmOrConnection != null) {
            builder.setGrantType(ParameterBuilder.GRANT_TYPE_PASSWORD_REALM).setRealm(realmOrConnection);
        }
        else {
            builder.setGrantType(ParameterBuilder.GRANT_TYPE_PASSWORD);
        }
        const parameters = builder.asDictionary();
        return this.loginWithToken(parameters);
    }
    loginWithOTP(mfaToken, otp) {
        const parameters = ParameterBuilder.newBuilder()
            .setGrantType(ParameterBuilder.GRANT_TYPE_MFA_OTP)
            .setParameter(AuthenticationAPIClient.MFA_TOKEN_KEY, mfaToken)
            .setParameter(AuthenticationAPIClient.ONE_TIME_PASSWORD_KEY, otp)
            .asDictionary();
        return this.loginWithToken(parameters);
    }
    userInfo(accessToken) {
        return this.profileRequest()
            .addHeader(AuthenticationAPIClient.HEADER_AUTHORIZATION, "Bearer " + accessToken);
    }
    createUser(email, password, username, connection) {
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.DB_CONNECTIONS_PATH)
            .appendPath(AuthenticationAPIClient.SIGN_UP_PATH)
            .build();
        const parameters = ParameterBuilder.newBuilder()
            .setParameter(AuthenticationAPIClient.USERNAME_KEY, username)
            .setParameter(AuthenticationAPIClient.EMAIL_KEY, email)
            .setParameter(AuthenticationAPIClient.PASSWORD_KEY, password)
            .setConnection(connection)
            .setClientId(this.getClientId())
            .asDictionary();
        const request = this.factory.POST(url, DatabaseUser, this.authErrorBuilder)
            .addParameters(parameters);
        return new DatabaseConnectionRequest(request);
    }
    signUp(email, password, username, connection) {
        const createUserRequest = this.createUser(email, password, username, connection);
        const authenticationRequest = this.login(email, password, connection);
        return new SignUpRequest(createUserRequest, authenticationRequest);
    }
    resetPassword(email, connection) {
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.DB_CONNECTIONS_PATH)
            .appendPath(AuthenticationAPIClient.CHANGE_PASSWORD_PATH)
            .build();
        const parameters = ParameterBuilder.newBuilder()
            .setParameter(AuthenticationAPIClient.EMAIL_KEY, email)
            .setClientId(this.getClientId())
            .setConnection(connection)
            .asDictionary();
        const request = this.factory.POST(url, this.authErrorBuilder)
            .addParameters(parameters);
        return new DatabaseConnectionRequest(request);
    }
    revokeToken(refreshToken) {
        const parameters = ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .setParameter(AuthenticationAPIClient.TOKEN_KEY, refreshToken)
            .asDictionary();
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.REVOKE_PATH)
            .build();
        return this.factory.POST(url, this.authErrorBuilder)
            .addParameters(parameters);
    }
    renewAuth(refreshToken) {
        const parameters = ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .setRefreshToken(refreshToken)
            .setGrantType(ParameterBuilder.GRANT_TYPE_REFRESH_TOKEN)
            .asDictionary();
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.TOKEN_PATH)
            .build();
        return this.factory.POST(url, Credentials, this.authErrorBuilder)
            .addParameters(parameters);
    }
    getProfileAfter(authenticationRequest) {
        const profileRequest = this.profileRequest();
        return new ProfileRequest(authenticationRequest, profileRequest);
    }
    token(authorizationCode, redirectUri) {
        const parameters = ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .setGrantType(ParameterBuilder.GRANT_TYPE_AUTHORIZATION_CODE)
            .setParameter(AuthenticationAPIClient.OAUTH_CODE_KEY, authorizationCode)
            .setParameter(AuthenticationAPIClient.REDIRECT_URI_KEY, redirectUri)
            .asDictionary();
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.TOKEN_PATH)
            .build();
        const request = this.factory.POST(url, Credentials, this.authErrorBuilder);
        request.addParameters(parameters);
        return new TokenRequest(request);
    }
    loginWithToken(parameters) {
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.OAUTH_PATH)
            .appendPath(AuthenticationAPIClient.TOKEN_PATH)
            .build();
        const requestParameters = ParameterBuilder.newBuilder()
            .setClientId(this.getClientId())
            .addAll(parameters)
            .asDictionary();
        return this.factory.authenticationPOST(url)
            .addAuthenticationParameters(requestParameters);
    }
    profileRequest() {
        const url = Uri.parse(this.auth0.getDomainUrl()).buildUpon()
            .appendPath(AuthenticationAPIClient.USER_INFO_PATH)
            .build();
        return this.factory.GET(url, UserInfo, this.authErrorBuilder);
    }
}
AuthenticationAPIClient.USERNAME_KEY = "username";
AuthenticationAPIClient.PASSWORD_KEY = "password";
AuthenticationAPIClient.EMAIL_KEY = "email";
AuthenticationAPIClient.OAUTH_CODE_KEY = "code";
AuthenticationAPIClient.REDIRECT_URI_KEY = "redirect_uri";
AuthenticationAPIClient.TOKEN_KEY = "token";
AuthenticationAPIClient.MFA_TOKEN_KEY = "mfa_token";
AuthenticationAPIClient.ONE_TIME_PASSWORD_KEY = "otp";
AuthenticationAPIClient.SIGN_UP_PATH = "signup";
AuthenticationAPIClient.DB_CONNECTIONS_PATH = "dbconnections";
AuthenticationAPIClient.CHANGE_PASSWORD_PATH = "change_password";
AuthenticationAPIClient.OAUTH_PATH = "oauth";
AuthenticationAPIClient.TOKEN_PATH = "token";
AuthenticationAPIClient.USER_INFO_PATH = "userinfo";
AuthenticationAPIClient.REVOKE_PATH = "revoke";
AuthenticationAPIClient.HEADER_AUTHORIZATION = "Authorization";
//# sourceMappingURL=authenticationAPIClient.js.map