var TextUtils = android.text.TextUtils;
var Log = android.util.Log;
import { Auth0Exception } from '../auth0Exception';
import { PasswordStrengthErrorParser } from './passwordStrengthErrorParser';
export class AuthenticationException extends Auth0Exception {
    constructor(infoOrString = undefined, statusCode = 0) {
        super(AuthenticationException.DEFAULT_MESSAGE);
        Object.setPrototypeOf(this, new.target.prototype);
        if (typeof infoOrString === 'object') {
            this.values = Object.assign({}, infoOrString);
            const codeValue = (this.values[AuthenticationException.ERROR_KEY] != null
                ? this.values[AuthenticationException.ERROR_KEY]
                : this.values[AuthenticationException.CODE_KEY]);
            this.code = codeValue != null ? codeValue : AuthenticationException.UNKNOWN_ERROR;
            if (this.values[AuthenticationException.DESCRIPTION_KEY] == null) {
                this.description = this.values[AuthenticationException.ERROR_DESCRIPTION_KEY];
                this.warnIfOIDCError();
                return;
            }
            const description = this.values[AuthenticationException.DESCRIPTION_KEY];
            if (typeof description === 'string') {
                this.description = description;
            }
            else if (this.isPasswordNotStrongEnough()) {
                const pwStrengthParser = new PasswordStrengthErrorParser(description);
                this.description = pwStrengthParser.getDescription();
            }
        }
        else {
            this.code = infoOrString != null ? AuthenticationException.NON_JSON_ERROR : AuthenticationException.EMPTY_BODY_ERROR;
            this.description = infoOrString != null ? infoOrString : AuthenticationException.EMPTY_RESPONSE_BODY_DESCRIPTION;
            this.statusCode = statusCode;
        }
    }
    warnIfOIDCError() {
        if ("invalid_request" === this.getCode() && (AuthenticationException.ERROR_OIDC_ACCESS_TOKEN === this.getDescription() || AuthenticationException.ERROR_OIDC_RO === this.getDescription())) {
            Log.w('AuthenticationAPIClient', "Your Auth0 Application is configured as 'OIDC Conformant' but this instance it's not. To authenticate you will need to enable the flag by calling Auth0#setOIDCConformant(true) on the Auth0 instance you used in the setup.");
        }
    }
    getCode() {
        return this.code != null ? this.code : AuthenticationException.UNKNOWN_ERROR;
    }
    getStatusCode() {
        return this.statusCode;
    }
    getDescription() {
        if (!TextUtils.isEmpty(this.description)) {
            return this.description;
        }
        if (AuthenticationException.UNKNOWN_ERROR === this.getCode()) {
            return `Received error with code ${this.getCode()}`;
        }
        return "Failed with unknown error";
    }
    getValue(key) {
        if (this.values == null) {
            return null;
        }
        return this.values[key];
    }
    isBrowserAppNotAvailable() {
        return "a0.browser_not_available" === this.code;
    }
    isInvalidAuthorizeURL() {
        return "a0.invalid_authorize_url" === this.code;
    }
    isInvalidConfiguration() {
        return "a0.invalid_configuration" === this.code;
    }
    isMultifactorRequired() {
        return "mfa_required" === this.code || "a0.mfa_required" === this.code;
    }
    isMultifactorEnrollRequired() {
        return "a0.mfa_registration_required" === this.code || "unsupported_challenge_type" === this.code;
    }
    isMultifactorTokenInvalid() {
        return "expired_token" === this.code && "mfa_token is expired" === this.description ||
            "invalid_grant" === this.code && "Malformed mfa_token" === this.description;
    }
    isMultifactorCodeInvalid() {
        return "a0.mfa_invalid_code" === this.code || "invalid_grant" === this.code && "Invalid otp_code." === this.description;
    }
    isPasswordNotStrongEnough() {
        return "invalid_password" === this.code && "PasswordStrengthError" === this.values[AuthenticationException.NAME_KEY];
    }
    isPasswordAlreadyUsed() {
        return "invalid_password" === this.code && "PasswordHistoryError" === this.values[AuthenticationException.NAME_KEY];
    }
    isPasswordLeaked() {
        return "password_leaked" === this.code;
    }
    isRuleError() {
        return "unauthorized" === this.code;
    }
    isInvalidCredentials() {
        return "invalid_user_password" === this.code || "invalid_grant" === this.code && "Wrong email or password." === this.description;
    }
    isAccessDenied() {
        return "access_denied" === this.code;
    }
    isLoginRequired() {
        return "login_required" === this.code;
    }
}
AuthenticationException.ERROR_KEY = "error";
AuthenticationException.CODE_KEY = "code";
AuthenticationException.DESCRIPTION_KEY = "description";
AuthenticationException.ERROR_DESCRIPTION_KEY = "error_description";
AuthenticationException.NAME_KEY = "name";
AuthenticationException.DEFAULT_MESSAGE = "An error occurred when trying to authenticate with the server.";
AuthenticationException.ERROR_OIDC_ACCESS_TOKEN = "OIDC conformant clients cannot use /oauth/access_token";
AuthenticationException.ERROR_OIDC_RO = "OIDC conformant clients cannot use /oauth/ro";
//# sourceMappingURL=authenticationException.js.map