var Intent = android.content.Intent;
var Uri = android.net.Uri;
var Log = android.util.Log;
import { OAuthManager } from './oauthManager';
import { ResponseType } from './responseType';
import { FlagChecker } from './flagChecker';
import { CallbackHelper } from './callbackHelper';
import { AuthenticationException } from '../authentication/authenticationException';
import { AuthorizeResult } from './authorizeResult';
export class WebAuthProvider {
    constructor(account) {
        this.account = account;
        this.values = {};
        this.scheme = "https";
        this.withResponseType(ResponseType.CODE);
        this.withScope(WebAuthProvider.SCOPE_TYPE_OPENID);
    }
    withState(state) {
        this.values[OAuthManager.KEY_STATE] = state;
        return this;
    }
    withNonce(nonce) {
        this.values[OAuthManager.KEY_NONCE] = nonce;
        return this;
    }
    withAudience(audience) {
        this.values[WebAuthProvider.KEY_AUDIENCE] = audience;
        return this;
    }
    withScheme(scheme) {
        const lowerCase = scheme.toLowerCase();
        if (scheme !== lowerCase) {
            Log.w(WebAuthProvider.TAG, "Please provide the scheme in lowercase and make sure it's the same configured in the intent filter. Android expects the scheme in lowercase");
        }
        this.scheme = scheme;
        return this;
    }
    withScope(scope) {
        this.values[WebAuthProvider.KEY_SCOPE] = scope;
        return this;
    }
    withConnectionScope(...connectionScope) {
        const concatScope = connectionScope.map((x) => x.trim()).join(' ');
        if (concatScope.length > 0) {
            this.values[WebAuthProvider.KEY_CONNECTION_SCOPE] = concatScope;
        }
        return this;
    }
    withResponseType(type) {
        let sb = '';
        if (FlagChecker.hasFlag(type, ResponseType.CODE)) {
            sb += `${OAuthManager.RESPONSE_TYPE_CODE} `;
        }
        if (FlagChecker.hasFlag(type, ResponseType.ID_TOKEN)) {
            sb += `${OAuthManager.RESPONSE_TYPE_ID_TOKEN} `;
        }
        if (FlagChecker.hasFlag(type, ResponseType.TOKEN)) {
            sb += `${WebAuthProvider.RESPONSE_TYPE_TOKEN}`;
        }
        this.values[OAuthManager.KEY_RESPONSE_TYPE] = sb.trim();
        return this;
    }
    withParameters(parameters) {
        for (const key in parameters) {
            const value = parameters[key];
            if (value != null) {
                this.values[key] = value.toString();
            }
        }
        return this;
    }
    withConnection(connectionName) {
        this.values[OAuthManager.KEY_CONNECTION] = connectionName;
        return this;
    }
    withCustomTabsOptions(options) {
        this.ctOptions = options;
        return this;
    }
    withPKCE(pkce) {
        this.pkce = pkce;
        return this;
    }
    static hasBrowserAppInstalled(packageManager) {
        const intent = new Intent(Intent.ACTION_VIEW, Uri.parse("https://auth0.com"));
        return intent.resolveActivity(packageManager) != null;
    }
    start(activity, callback) {
        WebAuthProvider.managerInstance = null;
        if (this.account.getAuthorizeUrl() == null) {
            const ex = new AuthenticationException({
                code: "a0.invalid_authorize_url",
                description: "Auth0 authorize URL not properly set. This can be related to an invalid domain."
            }, 0);
            callback.onFailure(ex);
            return;
        }
        if (!WebAuthProvider.hasBrowserAppInstalled(activity.getPackageManager())) {
            const ex = new AuthenticationException({
                code: "a0.browser_not_available",
                description: "No Browser application installed to perform web authentication."
            }, 0);
            callback.onFailure(ex);
            return;
        }
        const manager = new OAuthManager(this.account, callback, this.values);
        manager.setCustomTabsOptions(this.ctOptions);
        manager.setPKCE(this.pkce);
        WebAuthProvider.managerInstance = manager;
        const redirectUri = CallbackHelper.getCallbackUri(this.scheme, activity.getApplicationContext().getPackageName(), this.account.getDomainUrl());
        manager.startAuthorization(activity, redirectUri, 110);
    }
    static resume(intent) {
        if (WebAuthProvider.managerInstance == null) {
            Log.w(WebAuthProvider.TAG, "There is no previous instance of this provider.");
            return false;
        }
        const data = new AuthorizeResult(intent);
        const success = WebAuthProvider.managerInstance.resumeAuthorization(data);
        if (success) {
            WebAuthProvider.managerInstance = null;
        }
        return success;
    }
    static getInstance() {
        return WebAuthProvider.managerInstance;
    }
}
WebAuthProvider.TAG = 'WebAuthProvider';
WebAuthProvider.KEY_AUDIENCE = "audience";
WebAuthProvider.KEY_SCOPE = "scope";
WebAuthProvider.KEY_CONNECTION_SCOPE = "connection_scope";
WebAuthProvider.SCOPE_TYPE_OPENID = "openid";
WebAuthProvider.RESPONSE_TYPE_TOKEN = "token";
//# sourceMappingURL=webAuthProvider.js.map